#ifndef HWDECLS_H
#define HWDECLS_H
//-----------------------------------------------------------------------------
/* List of all hardware class declarations, to be used with the automatically
   generated AVR device classes. */
class HWPort;
class HWSpi;
class HWUart;
class HWAcomp;
class HWPrescaler;
class HWTimer0;
class HWTimer1;
class HWMcucr;
class HWExtIrq;
class HWTimer01Irq;
class HWAdmux;
class HWAd;
class HWMcucr;
class HWSpi;
//-----------------------------------------------------------------------------
#endif
